#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create detection diagrams
# Author  : Christian Borgelt
# History : 2012.??.?? file created
#           2013.05.01 simplified with a title dictionary
#           2013.05.08 changed scale of pu_* and pr_* diagrams
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from math    import log10
UTILDIR = join('..', 'util')
if UTILDIR not in path: path.insert(0, UTILDIR)
from id2par  import get_args, project

#-----------------------------------------------------------------------

#cmax = zmax = 9
cmax = zmax = 12

#-----------------------------------------------------------------------

def header (out, pro):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('picture PatSize; '
             +'PatSize = btex \\small pattern size~$z$\\hnorm etex;\n')
    out.write('picture AssSize; '
             +'AssSize = btex \\small assembly size~$z$\\hnorm etex;\n')
    out.write('picture Coins;   '
             +'Coins   = btex \\small coincidences~$c$\\hnorm etex;\n')
    out.write('picture LogPats; '
             +'LogPats = btex \\small '
             +'$\\log(\\mbox{\\#patterns})$\\hnorm etex;\n')
    #out.write('picture AvgPats; '
    #         +'AvgPats = btex \\small avg.~\\#patterns\\hnorm etex;\n')
    out.write('picture Rate;    '
             +'Rate    = btex \\small rate\\hnorm etex;\n\n')
    out.write('numeric expand;  expand = 1/3;\n')
    out.write('numeric e;       e      = 2.718281828;\n\n')
    out.write('vardef init (expr cat, ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n; n = 0;\n')
    out.write('  if     (cat = "pc"):\n')
    out.write('        zmin = -4; zmax = 3;\n')
    out.write('  elseif (cat = "pu") or (cat = "pr"):\n')
    out.write('        zmin = -3; zmax = 3;\n')
    out.write('  elseif (cat = "su") or (cat = "sr"):\n')
    out.write('        zmin = -1; zmax = 3;\n')
    out.write('  elseif (cat = "sp") or (cat = "sb")\n')
    out.write('  or     (cat = "ov") or (cat = "ur"):\n')
    out.write('        zmin = -3; zmax = 1;\n')
    out.write('  else: zmin =  0; zmax = 1; fi;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor        := 0.7white;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.invalid.max      := zmin-1.5;\n')
    out.write('  barchart.null.max         := zmin-0.5;\n')
    out.write('  barchart.set_units(30mm/'+str(zmax-1)
                                +', 30mm/'+str(cmax-1)
                                +', 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges(('+str(zmax-1)+',0.84,0.08), '
                                   +'('+str(cmax-1)+',0.84,0.08), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.1 *fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_ylabel(0, Coins);\n')
    out.write('  if (cat = "pc") or (cat = "pu") or (cat = "pr")\n')
    out.write('  or (cat = "su") or (cat = "sr"):\n')
    out.write('        barchart.draw_xlabel(0, PatSize);\n')
    out.write('  else: barchart.draw_xlabel(0, AssSize); fi\n')
    out.write('  if     (cat = "pc"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-4,-3,-2,-1,0,1,2,3);\n')
    out.write('  elseif (cat = "pu") or (cat = "pr"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-3,-2,-1,0,1,2,3);\n')
    out.write('  elseif (cat = "su") or (cat = "sr"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-1,0,1,2,3);\n')
    out.write('  elseif (cat = "sp") or (cat = "sb")\n')
    out.write('  or     (cat = "ov") or (cat = "ur"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-3,-2,-1,0,1);\n')
    out.write('  else:\n')
    out.write('    axes.z.alabel.offset := (1.0mm,6mm);\n')
    out.write('    barchart.draw_zlabel(0, Rate);\n')
    out.write('    barchart.draw_zticks(0,0.2,0.4,0.6,0.8,1);\n')
    out.write('  fi\n')
    out.write('  barchart.draw_xticks("2"')
    for i in range(3,zmax+1):
        s = str(i) if zmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_yticks("2"')
    for i in range(3,cmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')
    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(fname, 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),int(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

titles = { 'pc': ('pattern','spectrum'),
           'fn': ('false neg.','super'),
           'fx': ('false neg.','exact'),
           'rx': ('excess','neurons'),
           'rn': ('no detect','patterns'),
           'ro': ('all other','patterns'),
           'sp': ('superset','patterns'),
           'sb': ('subset','patterns'),
           'ov': ('overlap','patterns'),
           'ur': ('unrelated','patterns') }

#-----------------------------------------------------------------------

def diag (out, fid, cat, args, rfnid=0, size=0, coin=0):
    name = 'n%d_p%g_r%g_w%g_%c' \
         % (args['ncnt'],args['period'],args['rate'],
            args['width']*1000,args['target'])
    excl = readfile(join('data','pc_'+name+'.txt')).keys() \
           if cat in ['pr','sr'] else set()
    if cat != 'pc':
        name = 'n%d_p%g_r%g_j%g_w%g_%c' \
             % (args['ncnt'],args['period'],args['rate'],
                args['jitter']*1000,args['width']*1000,args['target'])
    name = cat+'_'+name
    if   cat == 'pc':
        top,bot = titles[cat]
        scale = (-4,3,'log')
    elif cat in ['fn','fx','rx','rn','ro']:
        top,bot = titles[cat]
        name += '%d' % rfnid
        scale = (0,1,'linear')
    elif cat in ['sp','sb','ov','ur']:
        top,bot = titles[cat]
        name += '%d' % rfnid
        scale = (-3,1,'log')
    elif cat in ['pu','pr','su','sr']:
        top   = '%d neurons' % size
        bot   = '%d coins.'  % coin
        if cat[1:] == 'r': name += str(rfnid)
        name += '_%d_%d' % (size,coin)
        scale = (-1 if cat[:1] == 's' else -3,3,'log')
    else: print('error: unknown category'); exit()
    vals = readfile(join('data',name+'.txt'))
    if cat == 'pc':
        pairs = [(z,c) for z in range(2,zmax+1)
                       for c in range(2,cmax+1)]
        vals  = dict([(p,v) for p,v in vals.items() if p in pairs])
    out.write('beginfig(' +str(fid) +'); % '+name+'\n')
    out.write('  init("'+cat+'", '
                +'btex '+top+'\\hnorm etex, '
                +'btex '+bot+'\\hnorm etex);\n')
    for z in range(2,zmax+1):   # traverse the group sizes (neurons)
        out.write('  set_data(')# start the next data row
        for c in range(2,cmax+1): # traverse the numbers of coincidences
            if c > 2: out.write(', ')
            x = vals[(z,c)] if (z,c) in vals else 0
            if scale[2] == 'log':
                x = log10(x) if x > 0 else scale[0]-1
            elif x <= scale[0]:
                x = scale[0]-1
            if (z,c) in excl:
                x = scale[0]-2
            out.write('%f' % (x if x <= scale[1] else scale[1]+1))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, 5, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids  = [ [ ( 'ncnt',   [ 100 ] ),
                 ( 'period', [   3 ] ),
                 ( 'rate',   [  20 ] ),
                 ( 'target', [ 'c' ] ),
                 ( {'jitter':0.005, 'width':0.005},
                   {'jitter':0.004, 'width':0.004},
                   {'jitter':0.003, 'width':0.003},
                   {'jitter':0.002, 'width':0.002} ) ] ]
    rfnids = range(10)
    sigs   = [(z,c) for z in [5,6,7,8,9] for c in [5,6,7,8,9]]
    fid    = 0
    with open(argv[1] if len(argv) > 1 else 'coconad.mp', 'w') as out:
        header(out, 2)
        for i in range(get_args(grids)):
            args = get_args(grids, i)
            diag(out, fid, 'pc', args); fid += 1
            for r in rfnids:
                for cat in ['fn','fx','rx','rn','ro','sp','sb','ov','ur']:
                    diag(out, fid, cat, args, r); fid += 1
            for z,c in sigs:
                diag(out, fid, 'pu', args, 0, z, c); fid += 1
                for r in rfnids:
                    diag(out, fid, 'pr', args, r, z, c); fid += 1
        args['jitter'] = args['width'] = 0.003
        for z,c in [(5,5),(6,6),(7,7),(8,8),(9,9)]:
            diag(out, fid, 'su', args, 0, z, c); fid += 1
            for r in [0,4,5]:
                diag(out, fid, 'sr', args, r, z, c); fid += 1
        footer(out)
